{ File   : RAW2WAV.dpr
  Author : Deniz Oezmen
  Created: 2004-05-31
  Changed: 2007-08-05

  Stupid batch converter for raw sound files.
}
program RAW2WAV;

{$APPTYPE CONSOLE}

uses
  SysUtils, StrUtils;

type
  TBuffer = array of Byte;

var
  RAWFile      : file;
  Frequency    : Longword;
  BitsPerSample,
  Channels     : Word;
  WAVHeader    : packed array[0..43] of Char =
    ( 'R',  'I',  'F',  'F', #$00, #$00, #$00, #$00,  'W',  'A',  'V',  'E',
      'f',  'm',  't',  ' ', #$10, #$00, #$00, #$00, #$01, #$00, #$01, #$00,
     #$22, #$56, #$00, #$00, #$22, #$56, #$00, #$00, #$01, #$00, #$08, #$00,
      'd',  'a',  't',  'a', #$00, #$00, #$00, #$00);

function Prefix(FileName: string): string;
var
  i: Longint;
begin
  FileName := ExpandFileName(FileName);

  i := Length(FileName) + 1;
  repeat
    Dec(i)
  until (i = 0) or (PosEx('.', FileName, i) > 0);

  if (i = 0) or (PosEx('\', FileName, i + 1) > 0) then
    Result := FileName
  else
    Result := Copy(FileName, 1, i - 1)
end;

function OpenRAWFile(var RAWFile: file; RAWName: string): Boolean;
begin
  FileMode := fmShareDenyWrite;
  AssignFile(RAWFile, RAWName);

  {$I-}
  Reset(RAWFile, 1);
  {$I+}
  OpenRAWFile := (IOResult = 0)
end;

procedure Convert(RAWName: string);
var
  BlockAlign    : Word;
  BytesPerSecond,
  Size          : Longword;
  Buffer        : TBuffer;
  WAVFile       : file;
begin
  Write(RAWName, ' => ');

  RAWName := Prefix(RAWName) + '.wav';

  WriteLn(RAWName);

  Size := FileSize(RAWFile) + 36;
  BlockAlign := Channels * (BitsPerSample div 8);
  BytesPerSecond := Frequency * BlockAlign;

  Move(Size, WAVHeader[4], 4);
  Move(Channels, WAVHeader[22], 2);
  Move(Frequency, WAVHeader[24], 4);
  Move(BytesPerSecond, WAVHeader[28], 4);
  Move(BlockAlign, WAVHeader[32], 2);
  Move(BitsPerSample, WAVHeader[34], 2);

  Dec(Size, 36);

  Move(Size, WAVHeader[40], 4);

  FileMode := fmOpenReadWrite;
  Assign(WAVFile, RAWName);
  
  {$I-}
  Rewrite(WAVFile, 1);
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn('  Write error.');
    Exit
  end;

  SetLength(Buffer, Size);
  BlockRead(RAWFile, Buffer[0], Size);
  BlockWrite(WAVFile, WAVHeader[0], SizeOf(WAVHeader));
  BlockWrite(WAVFile, Buffer[0], Size);

  Close(WAVFile)
end;

procedure SetParams;
var
  i: Byte;
  s: string;
begin
  Frequency := 22050;
  Channels := 1;
  BitsPerSample := 8;
  
  for i := 2 to ParamCount do
  begin
    s := ParamStr(i);
    case s[1] of
      'f', 'F': Frequency := StrToInt(Copy(s, 2, Length(s) - 1));
      'c', 'C': Channels := StrToInt(Copy(s, 2, Length(s) - 1));
      'b', 'B': BitsPerSample := StrToInt(Copy(s, 2, Length(s) - 1))
    else
      WriteLn('Unknown parameter "', s, '"')
    end
  end
end;

procedure ScanFiles(FileParam: string);
var
  FileName,
  Dir,
  RawName  : string;
  SearchRec: TSearchRec;
begin
  // get the directory information from FileParam
  Dir := ExtractFilePath(FileParam);

  // search for all files matching FileParam
  if FindFirst(FileParam, faAnyFile - faDirectory, SearchRec) = 0 then
  begin
    repeat
      FileName := SearchRec.Name;
      RAWName := Dir + FileName;

      // if this file is accessible, analyze it
      if OpenRAWFile(RAWFile, RAWName) then
      begin
        Convert(RAWName);
        Close(RAWFile)
      end
      else
        WriteLn(RAWName, ' not accessible')
    until FindNext(SearchRec) <> 0
  end
  else
    WriteLn(Format('No matches for "%s"', [FileParam]));

  FindClose(SearchRec)
end;

begin
  if ParamCount >= 1 then
  begin
    SetParams;
    ScanFiles(ParamStr(1))
  end
  else
    Writeln('Incorrect number of parameters.')
end.
